import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

public class SelectProject {
	public static void main(String[] args) {
		try {
			int whereIndex = Arrays.asList(args).indexOf("WHERE");
			String[] fieldsToDisplay = Arrays.copyOfRange(args, 1, whereIndex);
			String[] relations = Arrays.copyOfRange(args, whereIndex + 1, args.length);
			
			File doc = new File(args[0]);
			if (!doc.exists()) throw new Exception("File not found");
			BufferedReader br = new BufferedReader(new FileReader(doc));
			String line = br.readLine();
			String[] fields = line.split("\t");
			
			// map each field with their index so we can retrieve values when checking relations
			HashMap<String, Integer> fieldIndexes = new HashMap<>(); 
			for (int i = 0; i < fields.length; i++)
				fieldIndexes.put(fields[i], i);
			
			printRow(fieldsToDisplay);
			
			while ((line = br.readLine()) != null) {
				fields = line.split("\t");
				if (checkRelations(fields, relations, fieldIndexes)) {
					
					// only display fields that were typed in
					String[] validFields = new String[fieldsToDisplay.length];
					int i = 0;
					for (String s : fieldsToDisplay) {
						int index = fieldIndexes.get(s);
						validFields[i] = fields[index];
						i++;
					}
					printRow(validFields);
				}
			}
		}
		catch (Exception e) {
			System.out.println("Error!");
		}
	}
	
	// given the entries of a row, print it
	private static void printRow(String[] arr) {
		StringBuilder str = new StringBuilder();
		for (String s : arr)
			str.append(s).append("\t");
		System.out.println(str);
	}
	
	// given a list of fields and relations, check if this row should be displayed or not
	// throws an error if the relations are not valid
	private static boolean checkRelations(String[] fields, String[] relations, HashMap<String, Integer> fieldIndexes) throws Exception {
		
		for (String relation : relations) {
			String[] relationArray = relation.split("\\.");
			String fieldName = relationArray[0];
			String relationName = relationArray[1];
			String targetValue = relationArray[2];
			int fieldIndex = fieldIndexes.get(fieldName);
			String fieldValue = fields[fieldIndex];
			
			switch (relationName) {
			case "eq":
				if (isNumber(targetValue) && Integer.parseInt(fieldValue) != Integer.parseInt(targetValue)) return false;
				else if (!isNumber(targetValue) && !fieldValue.equals(targetValue)) return false;
				break;
			case "lt":
				if (isNumber(targetValue) && Integer.parseInt(fieldValue) >= Integer.parseInt(targetValue)) return false;
				else if (!isNumber(targetValue) && fieldValue.compareTo(targetValue) >= 0) return false;
				break;
			case "gt":
				if (isNumber(targetValue) && Integer.parseInt(fieldValue) <= Integer.parseInt(targetValue)) return false;
				else if (!isNumber(targetValue) && fieldValue.compareTo(targetValue) <= 0) return false;
				break;
			case "le":
				if (isNumber(targetValue) && Integer.parseInt(fieldValue) > Integer.parseInt(targetValue)) return false;
				else if (!isNumber(targetValue) && fieldValue.compareTo(targetValue) > 0) return false;
				break;
			case "ge":
				if (isNumber(targetValue) && Integer.parseInt(fieldValue) < Integer.parseInt(targetValue)) return false;
				else if (!isNumber(targetValue) && fieldValue.compareTo(targetValue) < 0) return false;
				break;
			default:
				throw new Exception("Invalid Relations");
					
			}	
		}
		return true;
	}
	
	//check if the string is considered a number
	private static boolean isNumber(String s) {
		try {
			Integer.parseInt(s);
			return true;
		}
		catch (Exception e) { return false; }
	}
}
